'- BLHeliTV_20250218.bas --------------------------------------- >>fst'25<< -
'
' - reads and displays telemetry data from BlHeli_32 brushless
'   electronic speed controllers (ESC)
' - can write measurements to logfile
' - values can be calibrated via file Calibrate.ini
' - Hardware:
'   - BLHeli_32 ESC; firmware 32.5 and above
'     with 'Auto Telemetry' set to ON
'   - TTL level Serial-USB Adapter
' - OS: Windows XP and above
' - Programming Language: Liberty Basic Booster (LBB) 3.21
'                         http://www.lbbooster.com
' + Update 2020-05-03:
'   - Fix: Skipped dataframes on slow PCs (rewrite of GetSerialData).
'   - Fix: Data folder name is now BLHeliTV instead of BLHeliView.
'   - Add: Calibration data is now displayed in mainwindow.
'          Calibration.ini can be edited while BLHeliTV is running.
'
' + Update 2020-12-08:
'   - Add: Maximum temperature displayed. 
'   - Add: Possibility to change between peak data for V, A, W between:
'          - peak values at maximum rpm  or
'          - V min, A max, W max - values
'          -> button toggles between alternatives.
'
' + Update 2024-04-01:
'   - Not  an April prank ;-)
'   - Add: Remove one minimum- and one maximum-value per 13 dataframes
'          for Volt, Ampere and RpM. 
'
' + Update 2025-02-12:
'   - Improve: Display field for Amps extended to three digits.
'              Display field for Watt extended to four digits.
'
' + Update 2025-02-18:
'   - Cosmetic changes at display when pressing Reset button.
'
'- V 20250218 --------------------------------- http://www.FrankSteinberg.de -


'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
[Variables]

   Global Volt$, Ampere$, Watt$, RpM$, RV$, RW$, Temperature$, mAh$, UserPath$, CalibFilePath$, LogFilePath$, LogStatus$, StatusText$, BtnPeakText$
   Global Volt0$, Ampere0$, Watt0$, RpM0$, RV0$, RW0$, Temperature0$, TempMax$, mAh0$
   Global Volt,  Ampere,  Watt,  RpM,  RV,  RW,  Temperature,  mAh, Samples, Poles, NrLogged, startTime, DataFrames
   Global Volt0, Ampere0, Watt0, RpM0, TempMax
   Dim PortNum$(256)  
   Dim Buf(9)
   Dim Calib(5)
   Dim Poles$(16)
   For i = 1 to 16 
     Poles$(i) = Str$(i * 2)
   Next i
   LogStatus$ = "N"
   BtnPeakText$ = "Peak:"
   Poles = 7
   Samples = 13
   Volt0 = 99.99  'fake high value, because we want to get the min value
    
   !UserPath$ = @usr$
   UserPath$ = UserPath$ + "BLHeliTV"
   CalibFilePath$ = UserPath$ + "\Calibrate.ini"

'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
[BuildGUI]

    NoMainWin

	WindowWidth=780
	WindowHeight=520
	UpperLeftX=Int((DisplayWidth-WindowWidth)/2)
	UpperLeftY=Int((DisplayHeight-WindowHeight)/2)

	Statictext #main.staticVdata,"--.--",10,100,150,50
	Statictext #main.staticVtext,"Volt",170,120,50,20
	Statictext #main.staticAdata,"---.--",250,100,180,50
	Statictext #main.staticAtext,"Ampere",430,120,70,20
	Statictext #main.staticWdata,"----.-",510,100,180,50
	Statictext #main.staticWtext,"Watt",690,120,50,20
	Statictext #main.staticRdata,"-----",10,240,150,50
	Statictext #main.staticRtext,"RpM",170,260,50,20

	Statictext #main.staticCtext,"C",170,380,50,20
	Statictext #main.staticCMAXtext,"C max",170,430,70,20
	Statictext #main.staticCdata," --",66,360,90,50
	Statictext #main.staticCMAXdata," --",66,410,90,50

	Statictext #main.staticRVdata,"-----",270,240,150,50
	Statictext #main.staticRVtext,"RpM/V",430,260,70,20
	Statictext #main.staticCOMtext,"Com Port:",660,20,90,18
	Statictext #main.staticPOLEStext,"Motor Poles:",524,20,120,18
	Statictext #main.statictTITLEtext," BLHeli Telemetry View",10,30,260,30
	Statictext #main.staticCALIBRATEtext,"",14,76,760,32
	Statictext #main.staticLOGtext,"Log to file:",384,20,120,18
	Statictext #main.staticMAHdata,"-----",270,380,150,50
	Statictext #main.staticMAHtext,"mAh",430,400,50,20
	Statictext #main.staticRWdata," -----",500,240,180,50
	Statictext #main.staticRWtext,"RpM/W",690,260,60,20
	Statictext #main.staticSTATUSdata,"Status Field",4,468,766,18
	Statictext #main.staticCERRORStext,"Checksum Errors:",560,420,130,20
	Statictext #main.staticRECdata,"     0",700,380,50,20
	Statictext #main.staticLOGGEDtext,"Logged Dataframes:",544,400,150,20
	Statictext #main.staticCERRORSdata,"     0",700,420,50,20
	Statictext #main.staticVPEAKdata,"--.--",10,160,150,50
	Statictext #main.staticVPEAKtext,"V peak",170,180,70,20
	Statictext #main.staticAPEAKdata,"---.--",250,160,180,50
	Statictext #main.staticAPEAKtext,"A peak",430,180,70,20
	Statictext #main.staticWPEAKtext,"W peak",690,180,70,20
	Statictext #main.staticWPEAKdata,"----.-",510,160,180,50
	Statictext #main.staticRPEAKdata,"-----",10,300,150,50
	Statictext #main.staticRPMPEAKtext,"peak",170,320,50,20
	Statictext #main.staticRVPEAKtext,"peak",430,320,50,20
	Statictext #main.staticRWPEAKtext,"peak",690,320,50,20
	Statictext #main.staticRVPEAKdata,"-----",270,300,150,50
	Statictext #main.staticRWPEAKdata," -----",500,300,180,50
	Statictext #main.staticLOGGEDdata,"     0",700,400,50,20
	Statictext #main.staticRECtext,"Received Dataframes:",530,380,160,20
	'Statictext #main.staticRESETtext,"Peak:",290,20,60,18
	Button #main.buttonPEAK,BtnPeakText$,ButtonPEAKclick,UL,294,20,70,18
    Combobox #main.comboPORT,PortNum$(),ComboPORTclick,660,40,90,80  
	Button #main.buttonRESET,"Reset",ButtonRESETclick,UL,294,40,70,25
	Button #main.buttonSTART,"S t a r t",ButtonSTARTclick,UL,382,40,60,25
	Button #main.buttonSTOP,"S t o p",ButtonSTOPclick,UL,446,40,60,25
	Combobox #main.comboPOLES,Poles$(),ComboPOLESclick,524,40,120,80
    
    Stylebits #main, _WS_SYSMENU, _WS_MAXIMIZEBOX or _WS_SIZEBOX, 0, 0
	Stylebits #main.staticSTATUSdata, _SS_SUNKEN, 0, 0, 0
    Stylebits #main.statictTITLEtext, _SS_SUNKEN, 0, 0, 0
    
    Open "  BLHeliTV      2025-02-18      >>fst'25<<" For window As #main
    
    #main "trapclose QuitMain"
    #main.comboPOLES, "selectindex "; 7
    #main.buttonRESET,"!setfocus"
    #main.buttonSTOP,"!disable" 
	#main.comboPORT "font ms_sans_serif 10"
	#main.staticVdata "!font Courier_New 36 bold"
	#main.staticVtext "!font Lucida_Console 14"
	#main.staticAdata "!font Courier_New 36 bold"
	#main.staticAtext "!font Lucida_Console 14"
	#main.staticWdata "!font Courier_New 36 bold"
	#main.staticWtext "!font Lucida_Console 14"
	#main.staticRdata "!font Courier_New 36 bold"
	#main.staticRtext "!font Lucida_Console 14"
	#main.comboPOLES "font ms_sans_serif 10"
	#main.staticCtext "!font Lucida_Console 14"
	#main.staticCMAXtext "!font Lucida_Console 14"
	#main.staticRVdata "!font Courier_New 36 bold"
	#main.staticRVtext "!font Lucida_Console 14"
	#main.staticCOMtext "!font Lucida_Console 12"
	#main.staticCtext "!font Lucida_Console 14"
	#main.staticCMAXtext "!font Lucida_Console 14"
	#main.staticCdata "!font Courier_New 36 bold"
	#main.staticCMAXdata "!font Courier_New 36"
	#main.staticPOLEStext "!font Lucida_Console 12"
	#main.statictTITLEtext "!font Lucida_Sans 17"
	#main.buttonSTART "!font ms_sans_serif 10"
	#main.staticLOGtext "!font Lucida_Console 12"
	#main.staticMAHdata "!font Courier_New 36 bold"
	#main.staticMAHtext "!font Lucida_Console 14"
	#main.staticRWdata "!font Courier_New 36 bold"
	#main.staticRWtext "!font Lucida_Console 14"
	#main.staticSTATUSdata "!font Fixedsys 9"
	#main.staticCERRORStext "!font Lucida_Console 10"
	#main.staticRECdata "!font Courier_New 10"
	#main.staticLOGGEDtext "!font Lucida_Console 10"
	#main.staticCERRORSdata "!font Courier_New 10"
	#main.staticCALIBRATEtext "!font Lucida_Console 8"
	#main.buttonSTOP "!font ms_sans_serif 10"
	#main.staticVPEAKdata "!font Courier_New 36"
	#main.staticVPEAKtext "!font Lucida_Console 14"
	#main.staticAPEAKdata "!font Courier_New 36"
	#main.staticAPEAKtext "!font Lucida_Console 14"
	#main.staticWPEAKtext "!font Lucida_Console 14"
	#main.staticWPEAKdata "!font Courier_New 36"
	#main.staticRPEAKdata "!font Courier_New 36"
	#main.staticRPMPEAKtext "!font Lucida_Console 14"
	#main.staticRVPEAKtext "!font Lucida_Console 14"
	#main.staticRWPEAKtext "!font Lucida_Console 14"
	#main.staticRVPEAKdata "!font Courier_New 36"
	#main.staticRWPEAKdata "!font Courier_New 36"
	#main.staticLOGGEDdata "!font Courier_New 10"
	#main.staticRECtext "!font Lucida_Console 10"
	'#main.staticRESETtext "!font Lucida_Console 12"
	#main.buttonRESET "!font ms_sans_serif 10"
    
'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
[ComPortsInit]

  ONCOMERROR

  Redim PortNum$(256)
  DataFrames = 0
  NumIndex = 0
  
  '*** Search for ComPorts available:
  For i = 1 to 256
    Try
      ComStr$ =  "COM";Str$(i);": baud=115200 parity=n data=8 stop=1 odsr=off octs=off idsr=off"
      Open ComStr$ For Random As #ComPort
      '* If opening fails, the following lines will not be executed 
      NumIndex = NumIndex + 1               'count available ports
      PortNum$(NumIndex) = "Com" + Str$(i)  'store highest available portnumber
      Close #ComPort                        ' close port
    End Try
  Next i

  '*** Errormessage and prog-end, if no ComPort:
  If NumIndex < 1 Then 
     Notice "No ComPort available; closing !"
     Close #main
     End
  End If 

  '*** Put available ComPorts to combobox:
  PortNum$(0) = "* Rescan *"                     'first item is allways 'reload'
  #main.comboPORT, "reload"                      'get data from array PortNum$(NumIndex)
  #main.comboPORT, "selectindex "; NumIndex + 1  'array begins at 0, so the highest portnumber is selected

'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
[ComPortOpen]
  
  '*** Open ComPort to use:
  #main.comboPORT, "selection? PortSelected$"
  Try
    Open PortSelected$;": baud=115200 parity=n data=8 stop=1 odsr=off octs=off idsr=off" For Random As #ComPort
   Catch
    Notice PortSelected$; " not available !"
    Goto [ComPortsInit]
  End Try
  '*** Add message to statusline:
  Call AddToStatus " | Connected to "; PortSelected$
  
  ONCOMERROR [ComErrorHandler]

'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
[Data]
  
  Scan
  #main.staticSTATUSdata, "!contents? Tmp$"
  If Right$(Tmp$, 16) <> "Waiting for data" Then Call AddToStatus " | Waiting for data"
  
  startTime = Time$("ms") 'get start time
  Do 
    If Time$("ms") - startTime > 2000 Then Goto [Data]
  Loop Until Lof(#ComPort)
  Call AddToStatus " | Got data"

'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
[Calibration]

  'Call DoCalibFile
 

'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
[MainLoop]
'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  '*** Get N samples from ESC over serial port:
  Call GetSerialData
  
  '*** Read current calibration data from ini file; create if not exist:
  If DataFrames > 0 Then Call DoCalibFile
  
  '*** Calculate averages, consider calibration and format values:
  Call CalculateValues
  
  '*** Show main values in mainscreen: 
  Call DisplayValues
  
  '*** Write values to logfile, :
  Call LogValuesToFile 

'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Goto [MainLoop]
'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


'#############################################################################
Sub GetSerialData

  Temperature = 0 
  Volt = 0
  Ampere = 0
  mAh = 0
  RpM = 0
  
  VMin = 999999
  VMax = 0 
  AMin = 999999
  AMax = 0
  RMin = 999999
  RMax = 0  
  
  Trash$ = input$(#ComPort, Lof(#ComPort))

  '*** Get average of 13 values (this gives a duration of 0.5 seconds):
  For i = 1 To Samples
 
    [Sample]
    '** Get time of looking for a data frame:
    startTime = time$("ms") 
    
    Try

      '** Wait for serial data with 2 sec. timeout: 
      Do Until Lof(#ComPort)                     'loop ends when windows serial buffer contains data
        !SYS "Sleep", 5                    :REM  'reduce processor load (the program is most tome inside this loop)
        durationTime = Time$("ms") - startTime   'measure time since start
        If durationTime > 2000 Then Goto [Data]  'time-out after two seconds without data
        Scan                                     'react on user input
      Loop 
 
      '** Wait for a time-gap after a 10-byte dataframe (BLHeli time-gap between dataframes ~30ms)
      Do
        RecBytes = Lof(#ComPort)                            'number of bytes in buffer before sleep
        !SYS "Sleep", 1                               :REM  'wait a short time
        durationTime = Time$("ms") - startTime              'measure time since start
        If durationTime > 2000 Then Goto [Data]             'time-out after two seconds without data
        Scan                                                'react on user input
        RecBytes2 = Lof(#ComPort)                           'number of bytes in buffer after sleep
      Loop Until (RecBytes = RecBytes2) AND (RecBytes2 > 9) 'end, when no new data and 10 bytes in buffer

      '** Use only the 10 newest (rightmost) bytes:
      If Lof(#ComPort) > 10 Then Trash$ = input$(#ComPort, Lof(#ComPort) - 10)

      '** Store 10 bytes to array:
      For DataByte = 0 To 9
        Buf(DataByte) = Asc(Input$(#ComPort, 1))
      Next DataByte

    Catch
    
      Goto [ComErrorHandler]
    
    End Try
   
    '** Check for CRC8-error:
    If CalcCRC8(8) <> Buf(9) Then
      Call AddToStatus " |" + Time$() + ": CRC Error"
     CRCErrors += 1
     #main.staticCERRORSdata, Using ("######", CRCErrors)
     !SYS "Sleep", 100
     Goto [Data]
    End If
  
    '** Get single values:
    VSing = (Buf(1) * 256 + Buf(2)) / 100
    ASing = (Buf(3) * 256 + Buf(4)) / 100
    RSing =  Buf(7) * 256 + Buf(8)
    
    '** Calculate Min and Max values:
    If VSing < VMin Then VMin = VSing 
    If VSing > VMax Then VMax = VSing 
    If ASing < AMin Then AMin = ASing 
    If ASing > AMax Then AMax = ASing 
    If RSing < RMin Then RMin = RSing 
    If RSing > RMax Then RMax = RSing 
  
    '** Calculate sum of values:
    Temperature = Temperature + Buf(0)   
    mAh         = mAh    +  Buf(5) * 256 + Buf(6)
    Volt        = Volt   + VSing
    Ampere      = Ampere + ASing
    RpM         = RpM    + RSing
  
    '** Get number of dataframes an display it:
    DataFrames += 1
    #main.staticRECdata, Using ("######", DataFrames)

  Next i  'get next sample
  
  '** Remove Min and Max values:
  Volt   = Volt - VMin - VMax
  Ampere = Ampere - AMin - AMax
  RpM    = RpM - RMin - RMax
  
  
End Sub

'#############################################################################
Sub CalculateValues

  '** Calculate average values:
  Temperature = Temperature / Samples
  Volt = Volt / (Samples - 2)
  Ampere = Ampere / (Samples - 2)
  mAh = mAh  / Samples
  RpM = RpM  / (Samples - 2) / Poles
 
   
  '** Process calibration values:
  If Calib(1) > 0 Then Volt = Volt * Calib(1)
  If Volt <> 0 Then Volt = Volt + Calib(2)
  If Calib(3) > 0 Then Ampere = Ampere * Calib(3)
  If Ampere <> 0 Then Ampere = Ampere + Calib(4)
  If Calib(5) > 0 Then RpM = RpM * Calib(5)   
   
  '** Round to 100 RpM:
  RpM$ = Using ("#####", RpM)
  RpM = Val(RpM$) * 100
  
  '** Get the calculated values:
  Watt = Volt * Ampere
  If Volt > 0 Then RV = RpM / Volt Else RV = 0
  If Watt > 0 Then RW = RpM / Watt Else RW = 0
    
End Sub

'#############################################################################
Sub DisplayValues


  '** Display temperatures:
  Temperature$ = Using ("###", Temperature)
  If Temperature0$ <> Temperature$ Then #main.staticCdata, Temperature$ : Temperature0$ = Temperature$
  '** Display voltage:
  Volt$ = Using ("##.##", Volt)
  If Volt0$ <> Volt$ Then #main.staticVdata, Volt$ : Volt0$ = Volt$
  '** Display current:
  Ampere$ = Using ("###.##", Ampere)
  If Ampere0$ <> Ampere$ Then #main.staticAdata, Ampere$ : Ampere0$ = Ampere$
  '** Display rounded RpM:
  RpM$ = Using ("#####", RpM)
  If RpM0$ <> RpM$ Then #main.staticRdata, RpM$ : RpM0$ = RpM$
  '** Display capacitance:
  mAh$ = Using ("#####", mAh)
  If mAh0$ <> mAh$ Then #main.staticMAHdata, mAh$ : mAh0$ = mAh$
  '** Display power:
  Watt$ = Using ("####.#", Watt)
  If Watt0$ <> Watt$ Then #main.staticWdata, Watt$ : Watt0$ = Watt$
  '** Display RpM per Volt:
  RV$ = Using ("#####", RV)
  If RV0$ <> RV$ Then #main.staticRVdata,RV$ : RV0$ = RV$
  '** Display Rpm per Watt:
  RW$ = Using ("######", RW)
  If RW0$ <> RW$ Then #main.staticRWdata,RW$ : RW0$ = RW$


  '** Display peak-values, if RpM is bigger than before:
  If RpM > RpM0 Then
    If BtnPeakText$ = "Peak:" Then             
      #main.staticVPEAKdata, Volt$
      #main.staticAPEAKdata, Ampere$
      #main.staticWPEAKdata, Watt$
    End If
    #main.staticRPEAKdata, RpM$
    #main.staticRVPEAKdata, RV$
    #main.staticRWPEAKdata, RW$
    RpM0 = RpM                      'store last RpM value in RpM0
  End If


  '** Calculate Temperature max values:
  If Temperature > TempMax Then 
    TempMax = Temperature      'store max current
    #main.staticCMAXdata, Temperature$     'display current temperature as new max value
  End If
 
  '** Calculate and display min/max values:
  If BtnPeakText$ = "Min/Max:" Then   
    If Volt < Volt0 Then
      Volt0 = Volt                    'store min voltage
      #main.staticVPEAKdata, Volt$    'display current voltage as new min value
    End If      
    If Ampere > Ampere0 Then 
      Ampere0 = Ampere                'store max current
      #main.staticAPEAKdata, Ampere$  'display current amps as new max value
    End If
    If Watt > Watt0 Then 
      Watt0 = Watt                    'store max power
      #main.staticWPEAKdata, Watt$    'display current power as new max value
    End If    
  End If

End Sub

'#############################################################################
Sub LogValuesToFile

  '** Store dataframes to logfile and display the counter,
  '   only store after user input and when motor rotation is detected:  
  If LogStatus$ = "Y" And RpM > 9 Then
    '* print values to logfile; separator sign is a TAB:
    Print #LogFile, Volt$; CHR$(9); Ampere$; CHR$(9); Watt$; CHR$(9); RpM$; CHR$(9); RV$; CHR$(9); RW$; CHR$(9); Temperature$; CHR$(9); mAh$
    '* Count and display number of logged dataframes:
    NrLogged += 1
    #main.staticLOGGEDdata, Using ("######", NrLogged)    
  End If
  
End Sub

'#############################################################################
Function CalcCRC8(BufMax)

   For j = 0 To BufMax
     CRC8 = CRC8 Xor Buf(j)
     For i = 0 To 7
       If CRC8 And &h80 Then
         '* masking off the left-most bit before shifting prevents the Overflow error:
         CRC8 = ((&h7F And CRC8) * 2) Xor 7
        Else
         CRC8 = CRC8 * 2
       End If
     Next i
   Next j
   CalcCRC8 = CRC8
   
End Function

'#############################################################################
Sub AddToStatus Text$

  StatusText$ = StatusText$ + Text$
  StatusText$ = Right$(StatusText$, 94)
  #main.staticSTATUSdata, StatusText$ 

End Sub 

'#############################################################################
Sub DoCalibFile

  Try
    result = -1
    Open CalibFilePath$ For Input As #CalibFile
    While Eof(#CalibFile) = 0
      Line Input #CalibFile, Line$
      Line$ = Trim$(Line$)
      If Instr(".+-0123456789", Left$(Line$, 1)) Then 
        i += 1
        Calib(i) = Val(Line$)
      End If
    Wend
    Close #CalibFile
    result = 0
  End Try

  If result = -1 then
   result = MkDir(UserPath$)
   Try
     Open CalibFilePath$ For Output As #CalibFile
     Print #CalibFile, "# Calibration file for program  BLHeli Telemetry View."
     Print #CalibFile, "# Name MUST be 'Calibrate.ini' and has to be in the folder '...\Documents\BLHeliTV'" 
     Print #CalibFile, ""
     Print #CalibFile, "# Multiplicator values will be multiplicated with the readings from esc;"
     Print #CalibFile, "# can be greater or smaller than zero. Example: 1.0023"
     Print #CalibFile, ""
     Print #CalibFile, "# Offset values will be added or subtracted (if negative) to the readings from esc."
     Print #CalibFile, "# Example: -2.3"
     Print #CalibFile, ""
     Print #CalibFile, "# 1. value = Voltage multiplicator:"
     Print #CalibFile, "1.0"
     Print #CalibFile, ""
     Print #CalibFile, "# 2. value = Voltage offset:"
     Print #CalibFile, "+0.0"
     Print #CalibFile, ""
     Print #CalibFile, "# 3. value = Current multiplicator:"
     Print #CalibFile, "1.0"
     Print #CalibFile, ""
     Print #CalibFile, "# 4. value = Current offset: "
     Print #CalibFile, "+0.0"
     Print #CalibFile, ""
     Print #CalibFile, "# 5. value = RpM multiplicator:"
     Print #CalibFile, "1.0"
     Close #CalibFile
   End Try
  End If

  If Calib(1) = 0 Then Calib(1) = 1
  If Calib(1) < 0 Then Calib(1) = Calib(1) * -1
  If Calib(3) = 0 Then Calib(3) = 1
  If Calib(3) < 0 Then Calib(3) = Calib(3) * -1
  If Calib(5) = 0 Then Calib(5) = 1
  If Calib(5) < 0 Then Calib(5) = Calib(5) * -1 
  
  #main.staticCALIBRATEtext, "Calibration values (see ...\Documents\BLHeliTV\Calibrate.ini) =  "; "V*"; Calib(1); ";  V+"; Calib(2); ";  A*"; Calib(3); ";  A+"; Calib(4); ";  RpM*"; Calib(5)  

End Sub

'#############################################################################
Sub DoLogFile

  result = MkDir(UserPath$)
  LogFilePath$ = UserPath$ + "\Log_" + Date$("yyyy-mm-dd_") + Str$(Time$("seconds")) + ".csv"
  Try
    Open LogFilePath$ For Output As #LogFile
   Catch
    Call AddToStatus " | Logfile creation failed"
    Notice "Can't open logfile ! " + LogFilePath$
    Exit Sub
  End Try
  Call AddToStatus " | Created: " + LogFilePath$
  Print #LogFile, "'BLHeli Telemetry View  -  LOGFILE'"
  Print #LogFile, "'Created    : "; Date$("yyyy-mm-dd"); "  "; Time$(); "'"
  Print #LogFile, "'Calibration: V*"; Calib(1); ";  V+"; Calib(2); ";  A*"; Calib(3); ";  A+"; Calib(4); ";  RpM*"; Calib(5)
  Print #LogFile, "'Motor Poles: "; Poles * 2; "'"
  Print #LogFile, "'Motor      : '"
  Print #LogFile, "'Propeller  : '"
  Print #LogFile, "'ESC        : '"
  Print #LogFile, ""
  Print #LogFile, "U[V]"; CHR$(9); "I[A]"; CHR$(9); "P[W]"; CHR$(9); "RpM"; CHR$(9); "RpM/V"; CHR$(9); "RpM/W"; CHR$(9); "T[C]"; CHR$(9); "C[mAH]"  

End Sub

'#############################################################################
Sub ComboPORTclick
  
   #main.comboPORT, "contents? Selection$"
   Try : Close #ComPort : End Try
   If Selection$ = "* Rescan *" Then Goto [ComPortsInit]
   Goto [ComPortOpen]

End Sub

'#############################################################################
Sub ComboPOLESclick

  #main.comboPOLES, "contents? Selection$"
  Poles = Val(Selection$) / 2

End Sub


'#############################################################################
Sub ButtonPEAKclick
  
  If BtnPeakText$ = "Peak:" Then 
     BtnPeakText$ = "Min/Max:"
     #main.staticVPEAKtext, "V min"
     #main.staticAPEAKtext, "A max"
     #main.staticWPEAKtext, "W max"
    Else
     BtnPeakText$ = "Peak:"
     #main.staticVPEAKtext, "V peak"
     #main.staticAPEAKtext, "A peak"
     #main.staticWPEAKtext, "W peak"
  End If

  Print #main.buttonPEAK, BtnPeakText$
  
  Call ButtonRESETclick

End Sub


'#############################################################################
Sub ButtonRESETclick

  #main.staticVPEAKdata,  " -.--"
  #main.staticAPEAKdata,  "  -.--" 
  #main.staticWPEAKdata,  "   -.-"      
  #main.staticRPEAKdata,  "    -"
  #main.staticRVPEAKdata, "    -"
  #main.staticRWPEAKdata, "     -"
  #main.staticCMAXdata, "  -"  
  TempMax = 0
  Volt0 = 99.99
  Ampere0 = 0
  Watt0 = 0
  RpM0 = 0

End Sub

'#############################################################################
Sub ButtonSTARTclick
  
  If LogStatus$ = "Y" Then
    LogStatus$ =  "P"
    #main.buttonSTART, "Resume"
    Exit Sub
  End If

  If LogStatus$ = "P" Then
    LogStatus$ =  "Y"
    #main.buttonSTART, "Pause"
    Exit Sub
  End If
  
  Call DoCalibFile
  
  Call DoLogFile
  
  LogStatus$ = "Y"
  #main.buttonSTART, "Pause"
  #main.buttonSTOP,"!enable"

End Sub

'#############################################################################
Sub ButtonSTOPclick
  
  #main.buttonSTOP,"!disable"
  #main.buttonSTART, "S t a r t"
  Call AddToStatus " | Dataframes logged: " + Str$(NrLogged)
  LogStatus$ = "N"
  NrLogged = 0
  #main.staticLOGGEDdata, Using ("######", NrLogged)
  Print #LogFile, ""
  If BtnPeakText$ = "Peak:" Then 
     Print #LogFile, "'Last peak-values:'"
    Else
     Print #LogFile, "'Last min/max-values:'"
  End If
  #main.staticVPEAKdata,  "!contents? Tmp$" : Line$ = Line$ + Tmp$ + CHR$(9)
  #main.staticAPEAKdata,  "!contents? Tmp$" : Line$ = Line$ + Tmp$ + CHR$(9)
  #main.staticWPEAKdata,  "!contents? Tmp$" : Line$ = Line$ + Tmp$ + CHR$(9)      
  #main.staticRPEAKdata,  "!contents? Tmp$" : Line$ = Line$ + Tmp$ + CHR$(9)
  #main.staticRVPEAKdata, "!contents? Tmp$" : Line$ = Line$ + Tmp$ + CHR$(9)
  #main.staticRWPEAKdata, "!contents? Tmp$" : Line$ = Line$ + Tmp$ + CHR$(9)
  #main.staticCMAXdata,   "!contents? Tmp$" : Line$ = Line$ + Tmp$ + CHR$(9)
  #main.staticMAHdata,    "!contents? Tmp$" : Line$ = Line$ + Tmp$ 
  Print #LogFile, Line$
  Close #LogFile

End Sub  

'#############################################################################
Sub QuitMain

  Try : Close #ComPort : End Try
  Try : Close #LogFile : End Try
  Close #main
  End
  
End Sub                                              

'#############################################################################
[ComErrorHandler]
  Call AddToStatus " | Com Error"
  Try : Close #ComPort : End Try
  Try : Close #LogFile : End Try
  !SYS "Sleep", 1000
  Goto [ComPortsInit]

